@@ -12,9 +12,6 @@ import xarray as xr
 import scipy as sp
 import scipy.stats
 
-import tqdm
-
-import bebi103
 import bokeh
 import skimage
 import skimage.feature
@@ -28,7 +25,9 @@ import scipy.ndimage
 import os
 import os.path as osp
 import copy
+import itertools
 
+from concurrent.futures import ThreadPoolExecutor
 from .utils import *
 
 
@@ -148,8 +147,8 @@ class WBImage:
         
     def __getitem__(self, key):
         
-        if len(key) == 1:
-            key = key + (0, 0)
+        if type(key) == int:
+            key = (key, 0, 0)
             
         if len(key) == 2:
             key = key + (0,)
@@ -197,7 +196,7 @@ class DotImage:
                  hybcycle=0,
                  position=0,
                  ref_image=None,
-                 z_use=[0],
+                 z_use=None,
                  ):
         
         '''
@@ -275,8 +274,8 @@ class DotImage:
                 self.z_use = np.arange(0, working_image.slices)
                 self.image = working_image
         else:
-            self.z_method = None
-            self.z_use = [0]
+            self.z_method = 'full'
+            self.z_use = np.arange(0, working_images.slices)
             self.image = working_image
         
         
@@ -347,16 +346,24 @@ class DotImage:
             
             return
         
-        image_nobg = []
-        image_bg   = []
         
-        for im in self.image:
-            nobg, bg = subtract_background_fft(im['image'], radius)
-            image_nobg.append(nobg)
-            image_bg.append(np.std(bg))
+        bgsub_pack = itertools.product(iter(self.image), [radius])
+        
+        def bgsub_map(pack):
+            return subtract_background_fft(pack[0]['image'], pack[1])
+        
+        
+        with ThreadPoolExecutor() as pool:
+            output = np.array( list(pool.map(bgsub_map, bgsub_pack)) )
+
+            
+        output = np.transpose(output, (1, 0, 2, 3))
+        
+        
+        self.image_nobg = WBImage(np.reshape(output[0], self.shape))
+        self.image_bg   = np.array([ np.std(b) for b in output[1] ])
+        del output
         
-        self.image_nobg = WBImage(np.reshape(image_nobg, self.shape))
-        self.image_bg   = image_bg
         
         self.bg_subtracted = True
         
@@ -374,10 +381,21 @@ class DotImage:
             self.image_nobg = copy.copy(self.image) # needed for zip() iterator below. otherwise throws an error.
             self.image_bg   = [ np.min(a['image']) for a in self.image_nobg ]
             
-        output_df = pd.DataFrame([])
+        #output_df = pd.DataFrame({
+        #    'Slice': pd.Series([], dtype=np.dtype('uint8')),
+        #    'Channel': pd.Series([], dtype=np.dtype('uint8')),
+        #    'Frame': pd.Series([], dtype=np.dtype('uint8')),
+        #    'HybCycle': pd.Series([], dtype=np.dtype('uint16')),
+        #    'Position': pd.Series([], dtype=np.dtype('uint8')),
+        #    'x': pd.Series([], dtype=np.dtype('uint16')),
+        #    'y': pd.Series([], dtype=np.dtype('uint16')),
+        #    'area': pd.Series([], dtype=np.dtype('uint8')),
+        #    'label': pd.Series([], dtype=np.dtype('int')),
+        #    'max_intensity': pd.Series([], dtype=np.dtype('uint16'))
+        #})
         
         local_max_images = []
-        
+        output_df_temp   = []
         
         if type(thresh_abs) == int:
             thresh_abs = np.repeat(thresh_abs, self.channels)
@@ -388,42 +406,52 @@ class DotImage:
         if len(thresh_abs) != self.channels or len(thresh_coeff) != self.channels:
             raise ValueError('find_local_maxes: threshold required for each image channels')
         
+        tc = [ thresh_coeff[im['channel']] for im in self.image ]
+        ta = [ thresh_abs[im['channel']]   for im in self.image ]
         
-        for nobg, bg, withbg in zip(self.image_nobg, self.image_bg, self.image):
-            # Pull up the thresholds for the current channel
-            ta = thresh_abs[nobg['channel']]
-            tc = thresh_coeff[nobg['channel']]
+        localmax_pack = itertools.product(zip(self.image_nobg, self.image_bg, self.image, tc, ta), [mask], [verbose])
+        
+        def localmax_map(pack):
+            im_data = pack[0]
+            mask    = pack[1]
+            verbose = pack[2]
+            local_maxes, dots_df = local_max_image_2(im_data[0]['image'], # self.image_nobg
+                                     im_data[1],          # self.image_bg
+                                     thresh_coeff=im_data[3],          # tc
+                                     thresh_abs=im_data[4],          # ta
+                                     intensity_image=im_data[2]['image'],
+                                     mask=mask,
+                                     verbose=verbose)
             
-            local_maxes, dots_df = local_max_image_2(nobg['image'], 
-                                                     bg, 
-                                                     thresh_coeff=tc,
-                                                     thresh_abs=ta,
-                                                     intensity_image=withbg['image'],
-                                                     mask=mask,
-                                                     verbose=verbose)
+            dots_df['Slice']   = im_data[0]['slice']
+            dots_df['Channel'] = im_data[0]['channel']
+            dots_df['Frame']   = im_data[0]['frame']
             
-            local_max_images.append(local_maxes)
-            
-            dots_df['Slice']   = nobg['slice']
-            dots_df['Channel'] = nobg['channel']
-            dots_df['Frame']   = nobg['frame']
-            
-            output_df = output_df.append(dots_df, ignore_index=True)
+            return local_maxes, dots_df
             
+        
+        with ThreadPoolExecutor() as pool:
+            packed_results = np.array( list(pool.map(localmax_map, localmax_pack)) )
+                        
+        local_max_images = [ p[0] for p in packed_results ]
+        output_df_temp   = [ p[1] for p in packed_results ]
+        
         self.local_max_image = WBImage(np.reshape(local_max_images, self.shape))
+        output_df            = pd.concat(output_df_temp)
         
         output_df['HybCycle'] = self.hybcycle
         output_df['Position'] = self.position
                 
         output_df = output_df.rename(columns={
                                              'coords': 'Coords',
-                                             'area': 'Area',
-                                             'label': 'Label',
                                              'max_intensity': 'Intensity',
                                              })
         
         try:
             output_df['Coords'] = output_df.apply(lambda df: np.ravel(df['Coords']), axis=1)
+            output_df['x'] = [r[0] for r in output_df['Coords']]
+            output_df['y'] = [r[1] for r in output_df['Coords']]
+            output_df = output_df.drop(columns='Coords')
         except:
             if verbose:
                 print('Found 0 dots on channel {}, slice {}'.format(channel))
@@ -569,61 +597,31 @@ def show_two_ims(im_1, im_2, titles=[None, None], interpixel_distances=[1, 1], m
         max_intensity = [np.max(im_1.astype(float)), np.max(im_2.astype(float))]
         print(max_intensity)
     
+    p_1 = imshow(im_1,
+                frame_height=400,
+                frame_width=400,
+                title=titles[0],
+                cmap=color_mapper,
+                interpixel_distance=interpixel_distances[0],
+                length_units='µm',
+                colorbar=colorbar,
+                min_intensity=min_intensity[0],
+                max_intensity=max_intensity[0]
+                )
+    p_2 = imshow(im_2,
+                frame_height=400,
+                frame_width=400,
+                title=titles[1],
+                cmap=color_mapper,
+                interpixel_distance=interpixel_distances[0],
+                length_units='µm',
+                colorbar=colorbar,
+                min_intensity=min_intensity[1],
+                max_intensity=max_intensity[1]
+                )
     
-    bebi103_vers = bebi103.__version__.split('.')
-
-    if int(bebi103_vers[2]) <= 41 and int(bebi103_vers[1]) == 0:
-
-        p_1 = bebi103.viz.imshow(im_1,
-                                 plot_height=400,
-                                 plot_width=400,
-                                 title=titles[0],
-                                 color_mapper=color_mapper,
-                                 interpixel_distance=interpixel_distances[0],
-                                 length_units='µm',
-                                 colorbar=colorbar,
-                                 min_intensity=min_intensity[0],
-                                 max_intensity=max_intensity[0]
-                                )
-        p_2 = bebi103.viz.imshow(im_2,
-                                 plot_height=400,
-                                 plot_width=400,
-                                 title=titles[1],
-                                 color_mapper=color_mapper,
-                                 interpixel_distance=interpixel_distances[0],
-                                 length_units='µm',
-                                 colorbar=colorbar,
-                                 min_intensity=min_intensity[1],
-                                 max_intensity=max_intensity[1]
-                                )
-        p_2.x_range = p_1.x_range
-        p_2.y_range = p_1.y_range
-        
-    else:
-        p_1 = bebi103.image.imshow(im_1,
-                                 frame_height=400,
-                                 frame_width=400,
-                                 title=titles[0],
-                                 cmap=color_mapper,
-                                 interpixel_distance=interpixel_distances[0],
-                                 length_units='µm',
-                                 colorbar=colorbar,
-                                 min_intensity=min_intensity[0],
-                                 max_intensity=max_intensity[0]
-                                )
-        p_2 = bebi103.image.imshow(im_2,
-                                 frame_height=400,
-                                 frame_width=400,
-                                 title=titles[1],
-                                 cmap=color_mapper,
-                                 interpixel_distance=interpixel_distances[0],
-                                 length_units='µm',
-                                 colorbar=colorbar,
-                                 min_intensity=min_intensity[1],
-                                 max_intensity=max_intensity[1]
-                                )
-        p_2.x_range = p_1.x_range
-        p_2.y_range = p_1.y_range
+    p_2.x_range = p_1.x_range
+    p_2.y_range = p_1.y_range
         
     
     return bokeh.layouts.gridplot([p_1, p_2], ncols=2)
